/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DeployerManager.java 2262 2008-01-28 14:08:30Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.deploy.impl.deployer;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.DeployerException;
import org.ow2.util.ee.deploy.api.deployer.IDeployer;
import org.ow2.util.ee.deploy.api.deployer.IDeployerManager;
import org.ow2.util.ee.deploy.api.deployer.UnsupportedDeployerException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This manager is managing different deployers. The deployer that will be used
 * is a deployer that is supporting a given IDeployable object. If there are
 * many deployers for the same type of Deployable, the first deployer that is
 * supporting the given deployable will be used.
 * @author Florent Benoit
 */
public final class DeployerManager implements IDeployerManager {

    /**
     * Name of the property thats defines the deployer's class. (optional)
     */
    public static final String DEPLOYER_FACTORY_CLASS_NAME = DeployerManager.class.getName();

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(DeployerManager.class);

    /**
     * List of deployers.
     */
    private List<IDeployer> deployers = null;


    /**
     * Create a new Manager of deployer.
     */
    public DeployerManager() {
        this.deployers = new ArrayList<IDeployer>();
    }


    /**
     * Register a new Deployer on this manager instance.
     * @param deployer the deployer to register
     */
    public void register(final IDeployer deployer) {
        if (deployer != null) {
            logger.debug("Registering deployer ''{0}''", deployer.getClass());
            deployers.add(deployer);
        }
    }

    /**
     * Unregister a Deployer from this manager instance.
     * @param deployer the given deployer to unregister.
     */
    public void unregister(final IDeployer deployer) {

        if (deployers.contains(deployer)) {
            logger.debug("Unregistering deployer ''{0}''", deployer);
            deployers.remove(deployer);
        } else {
            logger.warn("Cannot unregister the deployer ''{0}'' as it is not registered.", deployer);
        }
    }

    /**
     * Find the first available deployer that is supporting the given deployable.
     * @param deployable the given deployable to test.
     * @return the fist available deployer
     * @throws UnsupportedDeployerException if no deployer is found for the given deployable.
     */
    protected IDeployer findDeployerForDeployable(final IDeployable<?> deployable) throws UnsupportedDeployerException {
        for (IDeployer deployer : deployers) {
            if (deployer.supports(deployable)) {
                return deployer;
            }
        }
        throw new UnsupportedDeployerException("No deployer was found for the deployable '" + deployable + "'.");
    }

    /**
     * Deploy a given deployable by finding the first deployer that accept this type of deployable.
     * @param deployable the given deployable to deploy.
     * @throws DeployerException if it cannot be undeployed.
     * @throws UnsupportedDeployerException if no deployer is found for the given deployable.
     */
    public void deploy(final IDeployable<?> deployable) throws DeployerException, UnsupportedDeployerException {
        // find a deployer and deploy
        findDeployerForDeployable(deployable).deploy(deployable);
    }

    /**
     * Undeploy a given deployable by finding the first deployer that accept this type of deployable.
     * @param deployable the given deployable to undeploy.
     * @throws DeployerException if it cannot be undeployed.
     * @throws UnsupportedDeployerException if no deployer is found for the given deployable.
     */
    public void undeploy(final IDeployable<?> deployable) throws DeployerException, UnsupportedDeployerException {
        // find a deployer and undeploy
        findDeployerForDeployable(deployable).undeploy(deployable);
    }


    /**
     * Deploy a given deployable by finding the first deployer that accept this type of deployable.
     * @param deployable the given deployable to deploy.
     * @throws DeployerException if it cannot be undeployed.
     * @throws UnsupportedDeployerException if no deployer is found for the given deployable.
     */
    public void isDeployed(final IDeployable<?> deployable) throws DeployerException, UnsupportedDeployerException {
        // find a deployer and deploy
        findDeployerForDeployable(deployable).isDeployed(deployable);
    }
}
